{{
  Time Format Object

  These functions operate on two longs formatted as follows:

  Time = sec (0-59) + min (0-59) << 8 + hour (0-23) << 16  
  Date = day (1-7) + date (1-31) << 8 + month (1-12) << 16 + year (0-99) << 24

  Note that these octets (binary) correspond to the byte data (BCD) returned
  from a DS1307 RTC chip.

  When setting the time, the set field index corresponds to the DS1307 register
  in which the value is found:

  0=sec 1=min 2=hour 3=day of week 4=date 5=month 6=year-2000

  This object follows the methodology of the strfmt object and so can be used
  by multiple objects and cog threads without penalty.
}}

con

  'color bytes to set and clear emphasis on set format
  '
  emset = %1111
  emclr = %1000

  'style bits
  '
  st_hr_12       = %0000_0001
  st_show_sec    = %0000_0010
  st_show_day    = %0000_0100
  st_spell_month = %0000_1000
  st_date_ddmm   = %0001_0000
  st_year_2d     = %0010_0000

dat
{
  This is a dat instead of var so that only one instance will
  be used no matter how many places this object is used
}
style   byte  st_show_sec + st_show_day + st_spell_month
  
obj
  sf        : "strfmt"

pub getstyle

  return style

pub setstyle(value)

  style := value

pub timefmt(ptr, atime) | h

  h := hour(atime)
  if style & st_hr_12
    if h > 12
      h -= 12
    elseif h == 0
      h := 12
  ptr := fmt2(ptr, h)
  ptr := sf.ch(ptr, ":")
  ptr := fmt2(ptr, minute(atime))
  if style & st_show_sec
    ptr := sf.ch(ptr, ":")
    ptr := fmt2(ptr, second(atime))
  if style & st_hr_12
    if hour(atime) > 12
      ptr := sf.str(ptr, string(" PM"))
    else
      ptr := sf.str(ptr, string(" AM"))
  result := ptr

pub datefmt(ptr, adate)

  if style & st_show_day
    ptr := dayname(ptr, day(adate))
    ptr := sf.ch(ptr, " ")
  if style & st_date_ddmm
    ptr := datedatefmt(ptr, adate)
    ptr := datesep1fmt(ptr)
    ptr := datemonthfmt(ptr, adate)
  else
    ptr := datemonthfmt(ptr, adate)
    ptr := datesep1fmt(ptr)
    ptr := datedatefmt(ptr, adate)
  ptr := datesep2fmt(ptr)
  if not style & st_year_2d
    ptr := sf.str(ptr, string("20"))
  result := fmt2(ptr, year(adate))

pri datemonthfmt(ptr, adate)

  if style & st_spell_month
    result := monthname(ptr, month(adate))
  else
    result := fmt2(ptr, month(adate))
    
pri datedatefmt(ptr, adate)

  result := fmt2(ptr, date(adate))

pri datesep1fmt(ptr)
' separator between date and month
  if style & st_spell_month
    result := sf.ch(ptr, " ")
  else
    result := sf.ch(ptr, "-")

pri datesep2fmt(ptr)
' separator before year
  if style & st_spell_month
    if not (style & constant(st_date_ddmm | st_year_2d))
      ptr := sf.ch(ptr, ",") 
    result := sf.ch(ptr, " ")
  else
    result := sf.ch(ptr, "-")

{
  Note that when setting the clock, the seconds and day
  are always displayed regardless of style settings.
  Note how the temporary buffer is used to pass a
  formatted value to ifset() to conditionally wrap it
  in emphasis codes if it's selected.
}
pub setclkfmt(ptr, atime, adate, aset) | p0, h, buf[4]

  p0 := ptr
  h := hour(atime)
  if style & st_hr_12
    if h > 12
      h -= 12
    elseif h == 0
      h := 12
  ptr := sf.str(ptr, string($C, emclr))
  fmt2(@buf, h)
  ptr := ifset(ptr, @buf, aset == 2)

  ptr := sf.ch(ptr, ":")
  fmt2(@buf, minute(atime))
  ptr := ifset(ptr, @buf, aset == 1)
  ptr := sf.ch(ptr, ":")
  fmt2(@buf, second(atime))
  ptr := ifset(ptr, @buf, aset == 0)
  if style & st_hr_12
    ptr := sf.ch(ptr, " ")
    if hour(atime) > 12
      sf.str(@buf, string("PM"))
    else
      sf.str(@buf, string("AM"))
    ptr := ifset(ptr, @buf, aset == 2)

  ptr := sf.ch(ptr, " ")
  
  dayname(@buf, day(adate))
  ptr := ifset(ptr, @buf, aset == 3)
  ptr := sf.ch(ptr, " ")
  if style & st_date_ddmm
    datedatefmt(@buf, adate)
    ptr := ifset(ptr, @buf, aset == 4)
    ptr := datesep1fmt(ptr)
    datemonthfmt(@buf, adate)
    ptr := ifset(ptr, @buf, aset == 5)
  else
    datemonthfmt(@buf, adate)
    ptr := ifset(ptr, @buf, aset == 5)
    ptr := datesep1fmt(ptr)
    datedatefmt(@buf, adate)
    ptr := ifset(ptr, @buf, aset == 4)
  ptr := datesep2fmt(ptr)
  bytemove(@buf, string("20"), 2)
  fmt2(@buf+2, year(adate))
  result := ifset(ptr, @buf, aset == 6)

pri ifset(ptr, srcptr, doset)
  '
  if doset
    ptr := sf.str(ptr, string($C, emset))
  ptr := sf.str(ptr, srcptr)
  if doset
    ptr := sf.str(ptr, string($C, emclr))
  result := ptr

pub dayname(ptr, aday)
  '
  return sf.lstr(ptr, string("???SunMonTueWedThuFriSat") + aday * 3, 3)

pub monthname(ptr, amonth)
  '
  return sf.lstr(ptr, string("???JANFEBMARAPRMAYJUNJULAUGSEPOCTNOVDEC") + amonth * 3, 3) 

pri fmt2(ptr, value)
  '
  return sf.fdec(ptr, value, 2, 2)
{
  Utilities to extrand and set the individual octets of the formatted time and date longs
}
pub second(atime)
  '
  return atime & $FF

pub minute(atime)
  '
  return (atime >> 8) & $FF
  
pub hour(atime)
  '
  return (atime >> 16) & $FF

pub day(adate)
  '
  return adate & $FF

pub date(adate)
  '
  return (adate >> 8) & $FF

pub month(adate)
  '
  return (adate >> 16) & $FF

pub year(adate)
  '
  return (adate >> 24) & $FF

pub setsecond(atime, value)
  '
  return (atime & !$FF) | (value & $FF)

pub setminute(atime, value)
  '
  return (atime & !$FF00) | ((value & $FF) << 8)
  
pub sethour(atime, value)
  '
  return (atime & !$FF0000) | ((value & $FF) << 16)

pub setday(adate, value)
  '
  return (adate & !$FF) | (value & $FF)

pub setdate(adate, value)
  '
  return (adate & !$FF00) | ((value & $FF) << 8)

pub setmonth(adate, value)
  '
  return (adate & !$FF0000) | ((value & $FF) << 16)

pub setyear(adate, value)
  '
  return (adate & !$FF000000) | ((value & $FF) << 24)

   